#!/usr/bin/perl -w
# Copyright (C) 2007 Mandriva
#                    Mathieu RENARD <mrenard@mandriva.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package drakmsync;

use standalone;
use lib qw(/usr/lib/libDrakX);

use MDK::Wizard::Wizcommon;
use common;
use bluetooth;
use profiles;
use msynctool;
use Data::Dumper;

my $wiz = new MDK::Wizard::Wizcommon;

sub get_icon() { 'pda_section' }

sub create_profile {
	my $in = interactive->vnew;
	require wizards;
	my $already_in_wizard = defined $::WizardWindow;
		$wiz = wizards->new({

		defaultimage => get_icon(),
		name => N("Create a new syncronization profile"),
		pages => {
			welcome => {
				if_(!$::isInstall, no_back => 1),
				name => N("This wizard will help you to create a new syncronization profile."),
				next => "profile_name"
			},
			profile_name => {
				name => N("Please enter the syncronization profile name"),
				data => sub { [{ label => N("Profile name :"), val => \$wiz->{var}{profile_name}}] },
				complete => sub {
						if ( $wiz->{var}{profile_name} eq "" ){
							$in->ask_warn(N("Warning"),N("You didn't enter a name."));
							return 1;
						}
						if ( msynctool::get_groups_by_name($wiz->{var}{profile_name}) != undef ){
							$in->ask_warn(N("Warning"),N("The group already exist."));
							return 1;
						}
						print N("Syncronization profile name: "). $wiz->{var}{profile_name} ."\n";
						return 0;
        				},
				post  => sub { 
						$wiz->{var}{config} = profiles::get_profiles();
						return 0;
					},
				next => "select_config"
				},

			select_config => {
				name => N("Please select a preconfiguration"),
				data => sub { [
						{	label => N("Available configuration profiles:") . "\n",
							type => 'list', val => \$wiz->{var}{selected_config},	
							list => $wiz->{var}{config},
							format => sub { $_->{name} },
						}
					]},
				post  => sub { 
						$wiz->{var}{devices} = bluetooth::get_blth_devices();
						return 'select_blth_device' if $wiz->{var}{devices} ;
						return 'detect_blth_device';
					},
			},
			detect_blth_device => {
				name => N("Detecting Bluetooth devices"),
				data => sub { [	{label => 	N("No devices detected !")  . "\n",
											N("Please, turn on your bluetooth devices")  . "\n",
											N("and click 'next' to start the detection...")  . "\n",
						}]},
						
				post  => sub { 
						#TODO: add pulse progress bar and run get_blth_devices() in a thread
						$wiz->{var}{devices} = bluetooth::get_blth_devices();
						return 'select_blth_device' if $wiz->{var}{devices} ;
						return 'detect_blth_device';
					},
			},

			select_blth_device => {
				name => N("Please select a bluetooth device"),
				data => sub {[
						{	
							label => N("Detected devices:"),
							type => 'list', val => \$wiz->{var}{selected_blth_device},	
							list => $wiz->{var}{devices},
							format => sub { 
									return join(N(",\t Address : "),$_->{id},$_->{mac});
								},
						},   
					]},
				post  => sub { 
						#TODO: add pulse progress bar and run get_blth_services() in a thread
						$wiz->{var}{blth_services} = bluetooth::get_blth_services($wiz->{var}{selected_blth_device});
						return "summary" 
						#TODO: Notify user for error and go back to selected device
						#return "summary" if ($wiz->{var}{blth_services})
						#return "select_blth_device";
					},
			},
			summary => {
				name => N("Summary"),
				data => sub {[
						{	
							label => N("Profile Name: %s\nProfile Device: %s - [ %s ]\nBased on: %s", $wiz->{var}{profile_name}, $wiz->{var}{selected_blth_device}{id}, $wiz->{var}{selected_blth_device}{mac}, $wiz->{var}{selected_config}{name}),
						},
					]},
				post => sub {
						$wiz->{var}{group_path}=msynctool::add_group($wiz->{var}{profile_name});
						$wiz->{var}{selected_config_details}=profiles::get_profile_details_by_name(
							$wiz->{var}{selected_config}->{name});
						msynctool::add_members($wiz->{var}{group_path},
							$wiz->{var}{selected_config_details},
							$wiz->{var}{selected_blth_device}{mac},
							$wiz->{var}{blth_services}
							);
        				},
				end => 1,
			},
		},
	});
	eval { $wiz->process($in) };
	my $err = $@;
	if (!$already_in_wizard) {
		$::WizardWindow->destroy if $::WizardWindow;
		$::WizardTable->destroy if $::WizardTable;
		undef $::WizardTable;
		undef $::WizardWindow;
	}
	return if $err =~ /wizcancel/;
	die $err if $err;
	$err = $@;
	$in->ask_warn(N("Error"), N("An unexpected error has happened:"). "\n$err") if $err;
}

1;
