#!/usr/bin/perl
# Copyright (C) 2007 Mandriva
#                    Mathieu RENARD <mrenard@mandriva.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package msynctool;

use MDK::Common;
use XML::Simple;
use File::Find;
use File::Path;
use File::Basename;
use Data::Dumper;


my $group_path=$ENV{HOME}. "/.opensync/";
my $xml_header=qq(<?xml version="1.0"?>\n);

sub get_plugins{ #OK
	grep { !/Available\s+plugins:\s+/i } `msynctool --listplugins`;
}

sub get_groups{#OK
	my @groups;
	print "msynctool::get_groups() in: [".$group_path." ]\n";
	File::Find::find(
			sub {
				my $current_file = $File::Find::name;
				-f $current_file && /syncgroup.conf/i && push(@groups,get_group_infos($current_file));
			}, $group_path);
	return @groups;
}

sub get_groups_by_name{ #OK
	my($group_name)=@_;
	my @groups=get_groups();
	print "msynctool::get_groups_by_name(): Looking for $group_name\n";
	foreach my $group(@groups){
		if($group->{name} eq $group_name){
			print "msynctool::get_groups_by_name(): ".$group->{name}." found\n";
			return $group;	
		}
	}
	print "msynctool::get_groups_by_name(): ".$group_name ."not found\n";
	return undef;
}

sub get_group_infos{
	my($group_file)=@_;
	my $in_xml=new XML::Simple(KeyAttr=>[]);
	my $syncgroup=$in_xml->XMLin($group_file);
	if( ref $syncgroup->{groupname}){
		$syncgroup->{groupname}="N/A";
	}
	return { name=>$syncgroup->{groupname},last_sync=>$syncgroup->{last_sync},path=>dirname($group_file) };
}

sub add_group{
	my($group_name)=@_;
	my $default_group_path=$group_path.'group';
	my $i=1;
	my $out_xml=new XML::Simple(NoAttr=>1,RootName=>'syncgroup');
	
	#Check if group name exist 
	if(my $group=get_groups_by_name($group_name)){
		print "msynctool::add_group(): group ".$group_name ."already exist\n";
		return undef;
	}

	#prepare group configuration structure 
	my %group_infos=(groupname=>$group_name,last_sync=>0);

	#Create group directory
	while( -d "$default_group_path$i"){ 
		$i++;
	}

	my $group_path="$default_group_path$i";	
	mkdir $group_path;

	#write xml file(syncgroup.conf) 
	output_p("$group_path/syncgroup.conf", $xml_header.$out_xml->XMLout(\%group_infos))
	or die("Cannot Create Group File($group_path/syncgroup.conf)");

	print "msynctool::add_group(): Added group ".$group_name." [ ".$group_path." ]\n";
	return $group_path;
}

sub del_group{ #OK
	my($group_name)=@_;
	if(my $group=get_groups_by_name($group_name)){
		rmtree($group->{path});
		print "msynctool::del_group(): Removed group ".$group->{name}." [ ".$group->{path}." ]\n";
	}
}

sub sync_group{
	my($group_name)=@_;
	my $msynctool_options;
	
	#Get group info...
	my $group=get_groups_by_name($group_name) || die("Group $group_name does not exist");

	#Get plugin device address...
	my @plug_devices=map {
	    get_member_device_addr("$_->{path}/$_->{name}.conf");
	} get_members($group_name);

	#TODO: Check for device before syncing...

	#Read msynctool options files if exist
	my $options_file=$group->{path}."/options.conf";
	if(-f $options_file){
		print "Option file found: [ ".$options_file ." ]\n";
		my $options_xml=new XML::Simple(KeyAttr=>[]);
		my $options=$options_xml-> XMLin( $options_file) ;
		#FIXME Not an ARRAY reference...
		#if( @{$options->{option}} ) {
		#	$msynctool_options = join(" ", @{$options->{option}});
		#}
		#else{
			$msynctool_options=$options->{option} if (! ref $options->{option});
		#}
	}
	print "msynctool --sync ".$group_name." ".$msynctool_options."\n";
	my $msynctool_result=`msynctool --sync "$group_name" $msynctool_options`;
	print $msynctool_result."\n";
	return $msynctool_result;
}

sub get_members{
	my($group_name)=@_; 
	my @members;

	print "msynctool::get_members()\n";
	my $group=get_groups_by_name($group_name) || die("Group $group_name does not exist");	
	File::Find::find(
			sub {
				my $current_file = $File::Find::name;
			    -f $current_file && /syncmember.conf/i && push(@members,get_member_infos($current_file))
			}, $group->{path});
	return @members;
}

sub get_member_infos{ 
	my($member_file)=@_;
	print "msynctool::get_member_infos(), for ".$member_file."\n";
	my $syncmember_xml=new XML::Simple(KeyAttr=>[]);
	my $syncmember=$syncmember_xml -> XMLin( $member_file) ;
	return { name=>$syncmember->{pluginname}, path=>dirname($member_file) };
}

sub get_member_device_addr{
	my($conf_file)=@_;
	cat_($conf_file) =~ /((?:[0-9a-f]{2}[:-]){5}[0-9a-f]{2})/i;
}


sub add_members{ #OK
	my($group_path,$config,$device_mac,$device_services)=@_;
	my $i=1;
	my $plug_path;
	
	print "msynctool::add_member()\n";

	#Create msynctool options files
	my $options_xml=new XML::Simple(NoAttr=>1, RootName=>'options');
	output_p("$group_path/options.conf", $xml_header.$options_xml->XMLout($config->{options}))
	or die("Cannot Create msynctool Options File($group_path/options.conf)");

	foreach(@{$config ->{plugins}{plugin}}){
		my $service_channel;
		#Check for service on device and get the channel if needed
		if(! ref $_->{servicename}){
			print "msynctool::add_members(): Looking for ".$_->{servicename}." service on device\n";
			foreach my $service(@$device_services){
				next if $service->{protocol} !~ /$_->{servicename}\s*/i;
				print "msynctool::add_members(): ".$_->{servicename}." found on channel: ".$service->{channel}."\n";
				$service_channel=$service->{channel};
			}
			if(! $service_channel){
		 		print "msynctool::add_members(): Service ".$_->{servicename}." not found for selected device";
				return undef;
			}
		}
		
		#Create plugin directory
		while( -d "$group_path/$i"){ 
			$i++;
		}
		$plug_path="$group_path/$i";
		mkdir $plug_path;
		print "msynctool::add_members(): ".$_->{syncmember}{pluginname}." [ ".$plug_path." ]\n";

		#write xml plugin file(syncmember.conf)
		my $syncmember_xml=new XML::Simple(NoAttr=>1, RootName=>'syncmember');
		output_p("$plug_path/syncmember.conf", $xml_header.$syncmember_xml->XMLout($_->{syncmember}))
		or die("Cannot Create Plugin File($plug_path/syncmember.conf)");
	
		#write xml plugin configuration file(pluginname.conf)
		my $config_xml=new XML::Simple(NoAttr=>1, RootName=>'config');

		my $config_data=$config_xml->XMLout($_->{config});

		#TODO Put 2 lignes bellow in sub replace_keywords();
		$config_data=~ s/\[MAC_ADDRESS]/$device_mac/gi ;
		$config_data=~ s/\[PLUG_CHANNEL]/$service_channel/gi ;

		if($config_data !~ m!<config></config>!i) {
			output_p("$plug_path/$_->{syncmember}{pluginname}.conf", $config_data)
			or die("Cannot Create Plugin Configuration File($_->{syncmember}{pluginname}.conf)");
		}
		else{
			print 	"msynctool::add_members(): Warning! No configuration specified for". $_->{syncmember}{pluginname} ."\n";
		}
	}
	return $plug_path;
}

#sub replace_keywords{
#}

1;
